from django.contrib import admin
from .models import SourceDocument, DocumentChunk


@admin.action(description="Parse and index file")
def parse_and_index(modeladmin, request, queryset):
    for instance in queryset:
        instance.parse_file()


class SourceDocumentAdmin(admin.ModelAdmin):
    list_display = ["title", "indexed", "chunks_count"]
    actions = [parse_and_index]

    @admin.display(description="Number of chunks")
    def chunks_count(self, instance):
        return instance.documentchunk_set.count()


class DocumentChunkAdmin(admin.ModelAdmin):
    exclude = ["embedding", "index"]
    search_fields = ["page_content"]

    def get_search_results(self, request, queryset, search_term):
        if not search_term:
            return queryset, False

        results = queryset.intersection(self.model.objects.search(search_term))
        return results, False


admin.site.register(SourceDocument, SourceDocumentAdmin)
admin.site.register(DocumentChunk, DocumentChunkAdmin)



# Generated by Django 4.2.3 on 2023-07-11 13:55

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_langchain", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="documentchunk",
            name="index",
            field=models.IntegerField(),
        ),
    ]



import os
from django.db import models
from pgvector.django import VectorField, CosineDistance
from langchain.document_loaders import UnstructuredFileLoader
from langchain.docstore.base import Docstore
from langchain.embeddings.huggingface import HuggingFaceEmbeddings


embedding_model = HuggingFaceEmbeddings()


class SourceDocument(models.Model):
    title = models.CharField(blank=True, default="")
    file = models.FileField()
    indexed = models.BooleanField(default=False)

    def parse_file(self, compute_embeddings=True):
        loader = UnstructuredFileLoader(self.file.path, mode="elements")

        docs = []
        for index, doc in enumerate(loader.load(), start=1):
            docs.append(
                DocumentChunk(
                    index=index,
                    metadata=doc.metadata,
                    page_content=doc.page_content,
                    source=self,
                )
            )

        if compute_embeddings:
            for doc in docs:
                doc.embedding = embedding_model.embed_query(doc.page_content)

        DocumentChunk.objects.bulk_create(docs)
        self.indexed = True
        self.save()


class DocumentChunkManager(models.Manager):
    def search(self, query, max_results=5):
        query_embedding = embedding_model.embed_query(query)

        results = DocumentChunk.objects.order_by(
            CosineDistance("embedding", query_embedding)
        )[:max_results]

        return results


class DocumentChunk(models.Model):
    index = models.IntegerField()
    metadata = models.JSONField()
    page_content = models.TextField()
    embedding = VectorField(dimensions=768)
    source = models.ForeignKey(SourceDocument, on_delete=models.CASCADE)

    objects = DocumentChunkManager()

    def __str__(self):
        source = os.path.basename(self.metadata.get("source"))

        return f"[{source}][chunk:{self.index}] p{self.metadata.get('page_number')}"


class DocumentChunkStore(Docstore):
    def search(self, search: str):
        """
        Perform a similarity search throughout documents (i.e. text chunks)
        """
        chunk = DocumentChunk.objects.search(search)[0]
        return chunk.metadata.get("source"), chunk



# Generated by Django 4.2.3 on 2023-07-11 13:53

from django.db import migrations, models
import django.db.models.deletion
import pgvector.django


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        pgvector.django.VectorExtension(),
        migrations.CreateModel(
            name="SourceDocument",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(blank=True, default="")),
                ("file", models.FileField(upload_to="")),
                ("indexed", models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name="DocumentChunk",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("metadata", models.JSONField()),
                ("page_content", models.TextField()),
                ("embedding", pgvector.django.VectorField(dimensions=768)),
                (
                    "source",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_langchain.sourcedocument",
                    ),
                ),
                ("index", models.IntegerField(default=0)),
            ],
        ),
    ]



# coding: utf-8
from pathlib import Path
from setuptools import setup, find_packages


here = Path(__file__).parent
packages = find_packages("src")
main_package = packages[0]
long_description = (here / "README.md").read_text()
requirements = (here / "requirements.txt").read_text().split("\n")

setup(
    name="django-langchain",
    version="0.0.1",
    license="All rights reserved. 2023",
    description="Django Langchain",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=" P.A. SCHEMBRI",
    author_email="pa.schembri@advanced-stack.com",
    url="https://github.com/paschembri/django-langchain",
    packages=packages,
    package_dir={"": "src"},
    include_package_data=True,
    install_requires=requirements,
)
